/*  $Id: bootparam.h,v 1.15 1999/07/16 20:18:48 shaggy Exp $
 *      parameters passed from boot program to kernel 
 *      
 *      relative path: lib/boot/m-pc
 *  
 *  Originally written by Frank Barrus; hereby granted to the Public Domain
 *  with the hope that it may be useful.  Please document all changes to this
 *  file if it is redistributed, and, whenever reasonable, keep derviations of
 *  this work in the public domain, freely available to all who might benefit.
 *  THIS FILE IS PROVIDED "AS IS", WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 */

#include <btypes.h>

#define BootParam_ver_major	0
#define BootParam_ver_minor	96

#define MAX_PSEG	4	/* maximum number of program segments */



/* types of panics: (ix86 PC specific) */

#define PANIC_TYPE_IRQ		1	/* Interrupt Request 	*/
#define PANIC_TYPE_EXC		2	/* Exception		*/
#define PANIC_TYPE_RESET	3	/* trapped CPU reset	*/
#define PANIC_TYPE_OTHER	4	/* anything else	*/


#ifndef ASM
 
struct RegBlock;
struct TssBlock;
struct FileInfo;
struct CpuInfo;
struct MapFile;
struct VidMode;

#ifndef __cplusplus
	typedef struct RegBlock RegBlock;
	typedef struct TssBlock TssBlock;
	typedef struct FileInfo FileInfo;
	typedef struct CpuInfo CpuInfo;
	typedef struct MapFile MapFile;
	typedef struct PSeg PSeg;
	typedef struct BootParam BootParam;
	typedef struct VidMode VidMode;
#    ifndef bool
#	define bool	char
#    endif
#    ifndef true
#	define true	1
#    endif
#    ifndef false
#	define false	0	
#    endif
#endif

/* important note:
 *	If any changes are made to BootParam that 
 *	alter the offsets of existing attributes,
 *	then backwards compatability with older
 *	kernels or older boot loaders is lost.
 *	(for existing compiled binaries only)
 *	Therefore, unless absolutely necessary, all changes
 *	should be made in a manner that does not rearrange any
 *	attributes or change their sizes.
 *
 *	Especially don't change anything up to, and including the boot loader
 *	version and name, because it may be critical to obtain this
 *	information, to determinate the compatability of the interface.
 *
 *	Reserved spaces allow for future expansion without
 *	changing the layout extensively.
 *
 * Note: as of SOLO 0.98, the BootParam structure was changed
 * in an incompatible way.  I apologize for that, for anyone
 * who needed binary compatibility with the old format, but at
 * the time, the number of users seemed to be small, and the
 * changes were necessary to accomodate a more flexible structure
 * and thus ensure better future compatibility.
 */


struct PSeg {		// program segment
	_uint32	paddr;		// physical address (or file offset)
	_uint32	psize;		// physical size (or file seg size)
	_uint32	vaddr;		// virtual address
	_uint32	vsize;		// virtual size of segment
};

/* Note: MmSeg's can also be used to show free physical memory,
 * by giving them a vsize of 0
 */
struct MmSeg {		// mapped memory segment
	_uint32	paddr;		// physical address (or file offset)
	_uint32	psize;		// physical size (or file seg size)
	_uint32	vaddr;		// virtual address
	_uint32	vsize;		// virtual size of segment
	// need type, and type-dependent fields (pointers to strings, etc)
	// flags?
	// linked list, or array?
	// want to keep maximum flexibility here, so all future schemes
	// can be based on this
};

enum MmSegNum {
	MMSEG_TEXT = 0,
	MMSEG_DATA = 1,
	MMSEG_BSS = 2,
};

/* If we're using C, we can't scope these inside the structure... */
#ifndef __cplusplus
	typedef _uint16 index_t;
	typedef _uint16 fd_t;
#endif


/* BootParam interface format v0.96: */
struct BootParam {
#ifdef __cplusplus
	typedef	_uint16 index_t;/* file index type */
	typedef _uint16 fd_t;	/* file descriptor type */
#endif

	/* Generic, portable interface: */

	_uint8	bootver_major;	/* major number of boot loader version */
	_uint8	bootver_minor;	/* minor number of boot loader version */
	_uint8	bootparam_major;/* major number of bootparam version */
	_uint8	bootparam_minor;/* minor number of bootparam version */
	char	*bootname;	/* name of boot loader (without the version) */
	_int32	argc;		/* argc for main() */
	char	**argv;		/* *argv[] for main() */
	char	**envp;		/* *envp[] for main() (currently unused) */
	_uint32	reserved1;
	_uint32	reserved3;
	_uint32	entry;		/* starting execution address */
	_uint32	lomem;		/* lowest virtual address available */
	_uint32	himem;		/* highest virtual address available + 1 */
	/* move msgbuf up for compat, then change PSeg's to ptrs */
	/* maybe use for mapping other mem too? (type field?) */
	/* keep ptr to PSeg array, and size.. rename to MSeg */
	/* then need SOLO command for reserving segments */
	/* need 2nd set of ptrs to available memory */
	PSeg	text;		/* text segment */	
	PSeg	data;		/* data segment */
	PSeg	spare;		/* spare segment */
	PSeg	bss;		/* bss segment */
	PSeg	reserved;	/* future extra spare segment */
	_uint32	reserved15;
	_uint32	reserved16;

	_uint32	msgbuf;		/* kernel message buffer */
	_uint16	msgsize;	/* size of message buffer */
	_uint16	msgptr;		/* current pointer into message */

	fd_t	console_fd;	/* file descriptor for console */
	fd_t	self_fd;	/* file descriptor for current executable */
	_uint32	reserved21;
	void	(*exit)(_int32 code);		/* exit from kernel */
	void	(*reboot)(bool cold);		/* warm/cold reboot */
	fd_t	(*open)				/* open file and */
		(const char *name);		/* return 0 or fd */
	void	(*close)			/* close a file */
		(fd_t);
	const MapFile* (*readdir)		/* open directory entry and */
		(index_t index);		/* return 0 or map address */
	_int32	(*read)				/* read from a file/device */
		(fd_t, void *adr, _uint32 len);
	_int32	(*write)			/* write to a file/device */
		(fd_t, const void *adr, _uint32 len);
	void	(*fstat)
		(fd_t, FileInfo *stbuf);
	void	(*seek)				/* set file position */
#ifdef __cplusplus
		(fd_t, _uint32 ofs=0);
#else
		(fd_t, _uint32);
#endif
	_uint32	(*tell)				/* get file position */
		(fd_t);
	void	(*reserved25[2])(void);


	/* ix86 PC and PC BIOS Interface: */

	CpuInfo* cpuinfo;	/* CPU information */
	_uint32	buffer;		/* buffer for calling BIOS */
	_uint16	bufsize;	/* size of buffer */
	_uint8	bootdev;	/* BIOS boot device */
	_uint8	reserved4;
	_uint32	base_addr;	/* minimum base address */
	_uint32	base_size;	/* size of base memory */
	_uint32	base_lomem;	/* lowest available base address */
	_uint32	base_himem;	/* highest available base address + 1 */
	_uint32	ext_addr;	/* minimum extended memory address */
	_uint32	ext_size;	/* size of available extended memory */

	_uint16	timer0_count;	/* timer0 value */
	_uint8	timer0_mode;	/* timer0 mode */
	_uint8	reserved20;
	_uint32	reserved13;
	VidMode* vidmode;	/* VESA-style mode info (filled in for non-VESA 
				 * systems as best as possible)
				 */
	void	(*panic)	/* panic and exit */
		(_uint8 type, _uint32 num, _uint32 code,
		 TssBlock *tss);
		/* Note: panic() should exist on all platforms, but the
		 * parameters to it will vary depending on the nature of
		 * the information to be reported
		 */

	void	(*callbios)			/* call a BIOS interrupt */
		(_uint8 vec, RegBlock *reg);
	void	(*reserved26[2])(void);
};


/*

Kernel booting notes:

	After an executable is loaded from SOLO,
	virtual memory is initialized so that the executable
	is already mapped to where it wants to be loaded,
	and such that rest of that segment (up to 4MB) is
	also mapped in, giving some temporarily useable memory
	from 'lomem' to 'himem'.   This needs to be taken into
	account by any virtual memory management system that
	takes over afterwards.
	Also, physical memory from 00000000-000FFFFF is mapped
	into the same corresponding virtual addresses, allowing
	direct access to physical low memory with the same address.
	Therefore, the executable should *not* try to map
	itself into the low 1Mb of memory, or it will conflict
	with the mappings for physical memory.
	So, if addresses in this range are to be used, the old
	pagetable for low memory should be preserved, and must
	be used every time a call to any of the BootParam
	functions is made.  Or, a new mapping of the low 1Mb of memory
	must be set up.  Also, the first 32K of memory must not
	be altered, except in the regions from 
	(buffer to buffer+bufsize-1) and (msgbuf to msgbuf+msgsize-1)
	
*/



#else

#include "bootparam.ofs"

#endif

